@@ -221,7 +221,7 @@ struct i_get {
  * or case sensitive, so keep our own sanitized copy. */
 	char content[60];
 	char *charset;	/* extra content info such as charset */
-long long hcl;			/* http content length */
+curl_off_t hcl;			/* http content length */
 	char *cdfn;		/* http content disposition file name */
 	time_t modtime;	/* http modification time */
 	char *etag;		/* the etag in the header */
@@ -288,11 +288,13 @@ extern int verifyCertificates;	/* is a certificate required for the ssl connecti
 extern int displayLength;	// when printing a line
 extern int formatLineLength;	// when formatting html
 extern bool formatOverflow;
-extern int webTimeout, mailTimeout;
+extern long webTimeout, mailTimeout;
 extern uchar browseLocal;
 extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
 extern bool curlActive; // curl is running?
+extern bool netrc; // curl uses .netrc to resolve passwords
 extern const char *curlCiphers; // cipher string to send to curl
+extern char curlIPV; // 0 4 or 6
 extern bool ismc;		// Is the program running as a mail client?
 extern bool browseMail;		// Are we browsing a mail in html?
 extern bool passMail;		// pass mail across the filters
@@ -506,7 +508,7 @@ int ehsn;
 	bool irciMode:1;		// input irc
 	bool ircoMode:1;		// output irc
 	bool ircoMode1:1;		// was output irc
-	bool ircChannels:1; // show channel when displaying messages
+	bool showchan:1; // show channel when displaying messages
 	bool ircSecure:1; // ssl connection
 	bool ircPingOut:1; // sent ping to server
 	bool imapMode1:1;		// imap folders
