@@ -76,13 +76,15 @@ void scanUnreadMail(void);
 bool emailTest(void);
 void mail64Error(int err);
 char *emailParse(char *buf, bool plain);
-bool setupReply(bool all);
+bool setupReply(bool all, bool fwd);
 bool imapBufferPresent(void);
 bool imapBuffer(char *line);
 bool imap1rf();
 bool folderDescend(const char *path, bool rf);
 bool folderSearch(const char *path, char *search, bool rf);
 bool mailDescend(const char *path, char cmd);
+bool imapMovecopyG(char cmd, const char *uids, char *dest);
+bool imapDeleteG(const char *uids);
 bool imapMarkRead(int l1, int l2, char sign);
 bool imapMovecopy(int l1, int l2, char cmd, char *dest);
 bool imapDelete(int l1, int l2, char cmd);
@@ -213,7 +215,7 @@ void rowspan(void);
 // sourcefile=http.c
 void eb_curl_global_init(void);
 void eb_curl_global_cleanup(void);
-size_t eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get *g);
+size_t eb_curl_callback(char *incoming, size_t size, size_t nitems, void *data);
 time_t parseHeaderDate(const char *date);
 bool parseRefresh(char *ref, int *delay_p);
 bool shortRefreshDelay(const char *r, int d);
@@ -224,8 +226,9 @@ void *httpConnectBack3(void *ptr);
 void ebcurl_setError(CURLcode curlret, const char *url, int action, const char *curl_error);
 int ftpWrite(const char *url);
 void setHTTPLanguage(const char *lang);
+void curl_netrc(CURL *h);
 int prompt_and_read(int prompt, char *buffer, int buffer_length, int error_message, bool hide_echo);
-int ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data, size_t size, struct i_get *g);
+int ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data, size_t size, void *client);
 int bg_jobs(bool iponly);
 CURLcode setCurlURL(CURL * h, const char *url);
 bool frameExpand(bool expand, int ln1, int ln2);
@@ -254,7 +257,7 @@ bool loadAddressBook(void);
 const char *reverseAlias(const char *reply);
 bool encodeAttachment(const char *file, int ismail, bool webform, const char **type_p, const char **enc_p, char **data_p, bool *long_p);
 char *makeBoundary(void);
-bool sendMail(int account, const char **recipients, const char *body, int subjat, const char **attachments, const char *refline, int nalt, bool dosig);
+bool sendMail(int account, const char **recipients, const char *body, int subjat, const char **attachments, const char *refline, int nalt, int nfwd, bool dosig);
 bool validAccount(int n);
 bool sendMailCurrent(int sm_account, bool dosig);
 
@@ -279,6 +282,7 @@ void trimWhite(char *s) ;
 void stripWhite(char *s) ;
 void spaceCrunch(char *s, bool onespace, bool unprint) ;
 char *strmove(char *dest, const char *src) ;
+char *copyString(char *dest, const char *src, size_t n);
 char *initString(int *l) ;
 void stringAndString(char **s, int *l, const char *t) ;
 void stringAndBytes(char **s, int *l, const char *t, int cnt) ;
@@ -290,7 +294,6 @@ char *cloneString(const char *s) ;
 char *cloneMemory(const char *s, int n) ;
 void leftClipString(char *s) ;
 void shiftRight(char *s, char first) ;
-char *Cify(const char *s, int n) ;
 char *pullString(const char *s, int l) ;
 char *pullString1(const char *s, const char *t) ;
 int stringIsNum(const char *s) ;
