@@ -5,8 +5,9 @@
 #include <signal.h>
 #include <time.h>
 
-bool curlActive;
+bool curlActive, netrc;
 const char *curlCiphers;
+char curlIPV;
 int redirect_count = 0;
 char *serverData;
 int serverDataLen;
@@ -51,18 +52,9 @@ static void unlock_share(CURL * handle, curl_lock_data data, void *userptr)
 
 void eb_curl_global_init(void)
 {
-	const unsigned int major = 7;
-	const unsigned int minor = 29;
-	const unsigned int patch = 0;
-	const unsigned int least_acceptable_version =
-	    (major << 16) | (minor << 8) | patch;
-	curl_version_info_data *version_data = NULL;
 	curlActive = true;
 	CURLcode curl_init_status = curl_global_init(CURL_GLOBAL_ALL);
 	if (curl_init_status != 0) goto libcurl_init_fail;
-	version_data = curl_version_info(CURLVERSION_NOW);
-	if (version_data->version_num < least_acceptable_version)
-		i_printfExit(MSG_CurlVersion, major, minor, patch);
 
 // Initialize the global handle, to manage the cookie space.
 	global_share_handle = curl_share_init();
@@ -113,7 +105,7 @@ void eb_curl_global_cleanup(void)
 static void setup_download(struct i_get *g);
 static CURL *http_curl_init(struct i_get *g);
 static size_t curl_header_callback(char *header_line, size_t size, size_t nmemb,
-				   struct i_get *g);
+				   void *data);
 static bool ftpConnect(struct i_get *g, char *creds_buf);
 static bool gopherConnect(struct i_get *g);
 static bool dataConnect(struct i_get *g);
@@ -205,7 +197,10 @@ static void scan_http_headers(struct i_get *g, bool fromCallback)
 	}
 
 	if (!g->hcl && (v = find_http_header(g, "content-length"))) {
-		sscanf(v, "%lld", &g->hcl);
+                long long cl = 0;
+		sscanf(v, "%lld", &cl);
+// be explicit in case curl_off_t changes
+                g->hcl = (curl_off_t) cl;
 		nzFree(v);
 		if (g->hcl)
 			debugPrint(4, "content length %lld", g->hcl);
@@ -335,8 +330,9 @@ download states, in down_state:
 *********************************************************************/
 
 size_t
-eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get * g)
+eb_curl_callback(char *incoming, size_t size, size_t nitems, void *data)
 {
+        struct i_get *g = data;
 	size_t num_bytes = nitems * size;
 	int dots1, dots2, rc;
 
@@ -351,11 +347,9 @@ eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get * g)
 		if (g->hcl == 0) {
 // http should always set http content length, this is just for ftp.
 // And ftp downloading a file always has state = 1 on the first data block.
-			double d_size = 0.0;	// download size, if we can get it
 			curl_easy_getinfo(g->h,
-					  CURLINFO_CONTENT_LENGTH_DOWNLOAD,
-					  &d_size);
-			g->hcl = d_size;
+					  CURLINFO_CONTENT_LENGTH_DOWNLOAD_T,
+					  &g->hcl);
 			if (g->hcl < 0)
 				g->hcl = 0;
 		}
@@ -432,8 +426,8 @@ showdots:
  * All of the progress arguments to the function are unused. */
 
 static int
-curl_progress(void *data_p, double dl_total, double dl_now,
-	      double ul_total, double ul_now)
+curl_progress(void *data_p, curl_off_t dl_total, curl_off_t dl_now,
+	      curl_off_t ul_total, curl_off_t ul_now)
 {
 	struct i_get *g = data_p;
 	int ret = 0;
@@ -884,7 +878,7 @@ mimestream:
 
 	if (stringEqualCI(prot, "http") || stringEqualCI(prot, "https")) {
 		;		/* ok for now */
-	} else if (stringEqualCI(prot, "gopher")) {
+	} else if (stringEqualCI(prot, "gopher") || stringEqualCI(prot, "gophers")) {
 		return gopherConnect(g);
 	} else if (stringEqualCI(prot, "ftp") ||
 		   stringEqualCI(prot, "ftps") ||
@@ -972,7 +966,7 @@ mimestream:
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 	} else {
-		curlret = curl_easy_setopt(h, CURLOPT_HTTPGET, 1);
+		curlret = curl_easy_setopt(h, CURLOPT_HTTPGET, 1l);
 		if (curlret != CURLE_OK)
 			goto curl_fail;
 	}
@@ -1305,7 +1299,7 @@ they go where they go, so this doesn't come up very often.
 /* Convert POST request to GET request after redirection. */
 /* This should only be done for 301 through 303 */
 				if (g->code < 307) {
-					curl_easy_setopt(h, CURLOPT_HTTPGET, 1);
+					curl_easy_setopt(h, CURLOPT_HTTPGET, 1l);
 					post_request = false;
 				}
 /* I think there is more work to do for 307 308,
@@ -1718,7 +1712,7 @@ static void ftp_listing(struct i_get *g)
 }
 
 /* Format a line from a gopher directory. */
-static void gopher_ls_line(struct i_get *g, char *line)
+static void gopher_ls_line(struct i_get *g, char *line, const char *url_prot, const char *url_host)
 {
 	int port = 0;
 	char first, *text, *pathname, *host, *s, *plus;
@@ -1793,7 +1787,14 @@ static void gopher_ls_line(struct i_get *g, char *line)
 		} else {
 // Just a path
 			pathname = encodePostData(pathname, "./-_$");
-			stringAndString(&g->buffer, &g->length, "gopher://");
+                        if (stringEqualCI(url_host, host)) {
+/* If we've a listing then whether or
+ * not we're using TLS for this host it's working so stick with it. */
+                                stringAndString(&g->buffer, &g->length, url_prot);
+                                stringAndString(&g->buffer, &g->length, "://");
+                        }
+                        else
+                                stringAndString(&g->buffer, &g->length, "gopher://"); // default to plain gopher for compatibility
 			stringAndString(&g->buffer, &g->length, host);
 			if (port && port != 70) {
 				stringAndChar(&g->buffer, &g->length, ':');
@@ -1837,6 +1838,8 @@ static void gopher_listing(struct i_get *g)
 	char *s, *t;
 	char *incomingData = g->buffer;
 	int incomingLen = g->length;
+        char url_prot[MAXPROTLEN], url_host[MAXHOSTLEN];
+        getProtHostURL(g->url, url_prot, url_host);
 	g->buffer = initString(&g->length);
 	stringAndString(&g->buffer, &g->length, "<html>\n<body>\n");
 
@@ -1850,7 +1853,7 @@ static void gopher_listing(struct i_get *g)
 			if (!t || t >= incomingData + incomingLen)
 				break;	/* should never happen */
 			*t = 0;
-			gopher_ls_line(g, s);
+			gopher_ls_line(g, s, url_prot, url_host);
 			s = t + 1;
 		}
 	}
@@ -1981,15 +1984,13 @@ static bool ftpConnect(struct i_get *g, char *creds_buf)
 /* scp is somewhat unique among the protocols handled here */
 	is_scp = memEqualCI(url, "scp", 3);
 
-	if (stringEqual(creds_buf, ":") && memEqualCI(url, "ftp", 3))
-		strcpy(creds_buf, "anonymous:ftp@example.com");
-
 	h = http_curl_init(g);
-	if (!h)
-		goto ftp_transfer_fail;
-	curlret = curl_easy_setopt(h, CURLOPT_USERPWD, creds_buf);
-	if (curlret != CURLE_OK)
-		goto ftp_transfer_fail;
+	if (!h) goto ftp_transfer_fail;
+	if (!netrc && stringEqual(creds_buf, ":") && memEqualCI(url, "ftp", 3)) {
+		curlret = curl_easy_setopt(h, CURLOPT_USERPWD, 
+		"anonymous:ftp@example.com");
+		if (curlret != CURLE_OK) goto ftp_transfer_fail;
+	}
 
 	urlSanitize(g, 0);
 
@@ -2185,7 +2186,7 @@ fail:
 static bool gopherConnect(struct i_get *g)
 {
 	CURL *h;		// the curl handle for gopher
-	int protLength;		/* length of "gopher://" */
+	int protLength;		/* length of "gopher://" or "gophers://" */
 	bool transfer_success = false;
 	bool has_slash;
 	char first = 0;
@@ -2434,7 +2435,7 @@ my_curl_safeSocket(void *clientp, curl_socket_t socketfd, curlsocktype purpose)
 static CURL *http_curl_init(struct i_get *g)
 {
 	CURLcode curl_init_status = CURLE_OK;
-	int curl_auth;
+	long curl_auth;
 	CURL *h = curl_easy_init();
 	if (h == NULL)
 		goto libcurl_init_fail;
@@ -2446,6 +2447,7 @@ static CURL *http_curl_init(struct i_get *g)
 	curl_init_status = curl_easy_setopt(h, CURLOPT_COOKIEFILE, "");
 	if (curl_init_status != CURLE_OK)
 		goto libcurl_init_fail;
+	curl_netrc(h);
 // Lots of these setopt calls shouldn't fail.  They just diddle a struct.
 	curl_easy_setopt(h, CURLOPT_SOCKOPTFUNCTION, my_curl_safeSocket);
 	curl_easy_setopt(h, CURLOPT_WRITEFUNCTION, eb_curl_callback);
@@ -2453,21 +2455,21 @@ static CURL *http_curl_init(struct i_get *g)
 	curl_easy_setopt(h, CURLOPT_HEADERFUNCTION, curl_header_callback);
 	curl_easy_setopt(h, CURLOPT_HEADERDATA, g);
 	if (debugLevel >= 4)
-		curl_easy_setopt(h, CURLOPT_VERBOSE, 1);
+		curl_easy_setopt(h, CURLOPT_VERBOSE, 1l);
 	curl_easy_setopt(h, CURLOPT_DEBUGFUNCTION, ebcurl_debug_handler);
 	curl_easy_setopt(h, CURLOPT_DEBUGDATA, g);
-	curl_easy_setopt(h, CURLOPT_NOPROGRESS, 0);
-	curl_easy_setopt(h, CURLOPT_PROGRESSFUNCTION, curl_progress);
+	curl_easy_setopt(h, CURLOPT_NOPROGRESS, 0l);
+	curl_easy_setopt(h, CURLOPT_XFERINFOFUNCTION, curl_progress);
 	curl_easy_setopt(h, CURLOPT_PROGRESSDATA, g);
 	if(pubKey)
 		curl_easy_setopt(h, CURLOPT_SSH_PUBLIC_KEYFILE, pubKey);
 	curl_easy_setopt(h, CURLOPT_CONNECTTIMEOUT, webTimeout);
 	curl_easy_setopt(h, CURLOPT_USERAGENT, currentAgent);
-	curl_easy_setopt(h, CURLOPT_SSLVERSION, CURL_SSLVERSION_DEFAULT);
+	curl_easy_setopt(h, CURLOPT_SSLVERSION, (long)CURL_SSLVERSION_DEFAULT);
 /* We're doing this manually for now.
 	curl_easy_setopt(h, CURLOPT_FOLLOWLOCATION, allowRedirection);
 */
-	curl_easy_setopt(h, CURLOPT_AUTOREFERER, sendReferrer);
+	curl_easy_setopt(h, CURLOPT_AUTOREFERER, (long)sendReferrer);
 	if (ftpActive)
 		curl_easy_setopt(h, CURLOPT_FTPPORT, "-");
 	else
@@ -2496,6 +2498,10 @@ static CURL *http_curl_init(struct i_get *g)
 // set a smaller key size, which most servers don't care about, but some do.
 	if(curlCiphers)
 		curl_easy_setopt(h, CURLOPT_SSL_CIPHER_LIST, curlCiphers);
+	if(curlIPV == 4)
+		curl_easy_setopt(h, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
+	if(curlIPV == 6)
+		curl_easy_setopt(h, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V6);
 
 /* The next few setopt calls could allocate or perform file I/O. */
 	g->error[0] = '\0';
@@ -2515,6 +2521,15 @@ libcurl_init_fail:
 	return 0;
 }
 
+void curl_netrc(CURL *h)
+{
+	if(!netrc) return;
+	const char *n = getenv("NETRC");
+	if(n && *n)
+		curl_easy_setopt(h, CURLOPT_NETRC_FILE, n);
+	curl_easy_setopt(h, CURLOPT_NETRC, CURL_NETRC_OPTIONAL);
+}
+
 /*
  * There's no easy way to get at the server's response message from libcurl.
  * So here are some tables and a function for translating response codes to
@@ -2703,8 +2718,9 @@ static bool read_credentials(char *buffer)
  * Gather all the http headers into one long string. */
 static size_t
 curl_header_callback(char *header_line, size_t size, size_t nmemb,
-		     struct i_get *g)
+		     void *data)
 {
+        struct i_get *g = data;
 	const struct MIMETYPE *mt = 0;
 	size_t bytes_in_line = size * nmemb;
 	stringAndBytes(&g->headers, &g->headers_len,
@@ -2777,9 +2793,11 @@ nonascii:
 
 int
 ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
-		     size_t size, struct i_get *g)
+		     size_t size, void *client)
 {
 	FILE *f = debugFile ? debugFile : stdout;
+// We need to explicitly cast as apparently curl wants to pass us a void type pointer these days
+        struct i_get *g = (struct i_get *) client;
 
 
 // There's a special case where this function is used
@@ -2996,7 +3014,7 @@ CURLcode setCurlURL(CURL * h, const char *url)
 		curl_easy_setopt(h, CURLOPT_USERAGENT, agent);
 	}
 	curl_easy_setopt(h, CURLOPT_SSL_VERIFYPEER, verify);
-	curl_easy_setopt(h, CURLOPT_SSL_VERIFYHOST, (verify ? 2 : 0));
+	curl_easy_setopt(h, CURLOPT_SSL_VERIFYHOST, (long) (verify ? 2 : 0));
 // certificate file is per handle, not global, so must be set here.
 // cookie file is however on the global handle, go figure.
 	if (sslCerts)
@@ -3142,7 +3160,7 @@ bool frameExpand(bool expand, int ln1, int ln2)
 int frameExpandLine(int ln, Tag *t)
 {
 	pst line;
-	int tagno, start;
+	int tagno;
 	const char *s, *jssrc = 0;
 	char *a;
 	Frame *save_cf, *last_f;
@@ -3281,7 +3299,6 @@ So check for serverData null here. Once again we pop the frame.
 		createJSContext(cf);
 	nzFree(newlocation), newlocation = 0;
 
-	start = cw->numTags;
 	cdt = newTag(cf, "Document");
 	cdt->parent = t, t->firstchild = cdt;
 	cdt->attributes = allocZeroMem(sizeof(char*));
@@ -3348,7 +3365,7 @@ static int frameContractLine(int ln)
 
 bool reexpandFrame(void)
 {
-	int j, start;
+	int j;
 	Tag *frametag;
 	Tag *cdt;	// contentDocument tag
 	Frame *save_cf = cf;
@@ -3411,7 +3428,6 @@ bool reexpandFrame(void)
 	if (javaOK(cf->fileName))
 		createJSContext(cf);
 
-	start = cw->numTags;
 	debugPrint(3, "parse html from frame replace");
 	htmlScanner(serverData, cdt, false);
 	nzFree(serverData);	/* don't need it any more */
