@@ -12,7 +12,7 @@ Frame *cf;
 int gfsn; // global frame sequence number
 const char *progname;
 const char eol[] = "\r\n";
-const char *version = "3.8.12";
+const char *version = "3.8.13";
 char *changeFileName;
 char *configFile, *addressFile, *cookieFile, *emojiFile;
 char *mailDir, *mailUnread, *mailStash, *mailReply;
@@ -29,7 +29,7 @@ bool allowRedirection = true, allowJS = true, sendReferrer = true;
 bool blockJS;
 bool ftpActive;
 bool errorExit;
-int webTimeout = 20, mailTimeout = 0;
+long webTimeout = 20, mailTimeout = 0;
 int displayLength = 500;
 int verifyCertificates = 1;
 char *sslCerts, *pubKey;
@@ -488,9 +488,13 @@ int main(int argc, char **argv)
 
 	curlCiphers = getenv("EBCIPHERS");
 	if(curlCiphers && !*curlCiphers) curlCiphers = 0;
-// 1 is shorthand for the only seting we know of so far
+// 1 is shorthand for the only setting we know of so far
 	if(curlCiphers && stringEqual(curlCiphers, "1")) curlCiphers = "DEFAULT@SECLEVEL=1";
 
+	const char *ipv = getenv("EBIPV");
+	if(stringEqual(ipv, "4")) curlIPV = 4;
+	if(stringEqual(ipv, "6")) curlIPV = 6;
+
 // Establish the home directory, and standard edbrowse files thereunder.
 	home = getenv("HOME");
 
@@ -508,7 +512,7 @@ int main(int argc, char **argv)
 	if (fileTypeByName(configFile, 0) == 0) {
 		int fh = creat(configFile, MODE_private);
 		if (fh >= 0) {
-			ignore = write(fh, ebrc_string, strlen(ebrc_string));
+			write(fh, ebrc_string, strlen(ebrc_string));
 			close(fh);
 			i_printfExit(MSG_Personalize, configFile);
 		}
@@ -697,7 +701,7 @@ int main(int argc, char **argv)
 // These don't get freed, but we're going to exit anyways.
 
 		if (sendMail(account, (const char **)reclist2, body, 1,
-			     (const char **)atlist2, 0, nalt, true))
+			     (const char **)atlist2, 0, nalt, 0, true))
 			exit(0);
 		showError();
 		exit(1);
@@ -1379,6 +1383,7 @@ void unreadConfigFile(void)
 
 	memset(accounts, 0, sizeof(accounts));
 	maxAccount = localAccount = 0;
+	netrc = false;
 	memset(mimetypes, 0, sizeof(mimetypes));
 	maxMime = 0;
 	memset(dbtables, 0, sizeof(dbtables));
@@ -1921,11 +1926,11 @@ inside:
 			continue;
 
 		case 33:	// webtimer
-			webTimeout = atoi(v);
+			webTimeout = strtol(v, NULL, 10);
 			continue;
 
 		case 34:	// mailtimer
-			mailTimeout = atoi(v);
+			mailTimeout = strtol(v, NULL, 0);
 			continue;
 
 		case 35:	// certfile
@@ -2115,6 +2120,11 @@ nokeyword:
 			continue;
 		}
 
+		if (stringEqual(s, "netrc") && !(mailblock | mimeblock | tabblock | nest)) {
+			netrc = true;
+			continue;
+		}
+
 		if (*s == '\x82' && s[1] == 0) {
 			if (mailblock == 1) {
 				++maxAccount;
@@ -2123,9 +2133,9 @@ nokeyword:
 					cfgLine0(MSG_EBRC_NoInserver);
 				if (!act->outurl)
 					cfgLine0(MSG_EBRC_NoOutserver);
-				if (!act->login)
+				if (!act->login && !netrc)
 					cfgLine0(MSG_EBRC_NoLogin);
-				if (!act->password)
+				if (!act->password && !netrc)
 					cfgLine0(MSG_EBRC_NPasswd);
 				if (!act->from)
 					cfgLine0(MSG_EBRC_NoFrom);
