@@ -862,4 +862,17 @@ MSG_ImaskOn,
 	MSG_RestoringBOM,
 	MSG_SearchSameModeOff,
 	MSG_SearchSameModeOn,
+	MSG_DelDotDot,
+	MSG_ManyForward,
+	MSG_AltForward,
+	MSG_ForwardFirst,
+	MSG_NotEmail,
+	MSG_FwdDir,
+	MSG_FwdDB,
+	MSG_FwdIrc,
+	MSG_FwdImap,
+	MSG_FwdEmpty,
+	MSG_FwdBinary,
+	MSG_NoUnformat,
+	MSG_NoRaw,
 };
