@@ -34,7 +34,6 @@ static time_t adbooktime;
 bool loadAddressBook(void)
 {
 	char *buf, *bufend, *v, *record_start, *email_start, *at_sign, *dot, *full_name_start, *s, *t;
-	bool cmt = false;
 	enum { alias, local_part, quoted_part, backslash_escaped, domain,
 		full_name, ignore } state = alias;
 	char c;
@@ -490,7 +489,7 @@ empty:
 						setError(MSG_SigAccess);
 						goto freefail;
 					}
-					ignore = read(fd, buf + buflen, n);
+					read(fd, buf + buflen, n);
 					close(fd);
 					buflen += n;
 					buf[buflen] = 0;
@@ -499,7 +498,7 @@ empty:
 		}		/* .signature */
 	}
 
-	/* Infer content type from the filename */
+	// Infer content type from the filename
 	ct = 0;
 	s = strrchr(file, '.');
 	if (s && s[1]) {
@@ -516,6 +515,8 @@ empty:
 			ct = "video/mpeg";
 		if (stringEqualCI(s, "rtf"))
 			ct = "text/richtext";
+		if (stringEqualCI(s, "eml"))
+			ct = "message/rfc822";
 		if (stringEqualCI(s, "htm") ||
 		    stringEqualCI(s, "html") ||
 		    stringEqualCI(s, "shtm") ||
@@ -523,7 +524,7 @@ empty:
 			ct = "text/html";
 	}
 
-// alternative from a buffer is usually html; this doesn't fly if wev
+// alternative from a buffer is usually html; this doesn't fly if we
 // send it over as plain text. This is a crude test.
 // Just look for a leading <
 	if(!strncmp(file, "<session ", 9)) {
@@ -535,6 +536,12 @@ empty:
 			ct = "text/html";
 	}
 
+// Another simple test, for raw email format, again, probably too simple.
+// I wanted to use emailTest in fetchmail.c but that assumes the current buffer.
+	if(!ct && (!strncmp(buf, "Return-Path:", 12) ||
+	!strncmp(buf, "Delivered-To:", 13)))
+		ct = "message/rfc822";
+
 /* Count the nonascii characters */
 	nacount = nullcount = linelength = wordlength = 0;
 	longline = longword = cr = endlinespace = startlinedot = flowed = false;
@@ -823,10 +830,12 @@ struct smtp_upload {
 	int pos;
 };
 
+// Curl warns if we don't pass void pointers as data params so cast in the function
 static size_t smtp_upload_callback(char *buffer_for_curl, size_t size,
-				   size_t nmem, struct smtp_upload *upload)
+				   size_t nmem, void *data)
 {
-	size_t out_buffer_size = size * nmem;
+	struct smtp_upload *upload = (struct smtp_upload *) data;
+        size_t out_buffer_size = size * nmem;
 	size_t remaining = upload->length - upload->pos;
 	size_t to_send, cur_pos;
 
@@ -848,8 +857,7 @@ static char *buildSMTPURL(const struct MACCOUNT *account)
 {
 	char *url = NULL;
 	const char *scheme;
-	const char *smlogin = strchr(account->login, '\\');
-
+	const char *smlogin = account->login ? strchr(account->login, '\\') : 0;
 	if (smlogin)
 		++smlogin;
 	else
@@ -910,6 +918,10 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 
 	if(curlCiphers)
 		curl_easy_setopt(handle, CURLOPT_SSL_CIPHER_LIST, curlCiphers);
+	if(curlIPV == 4)
+		curl_easy_setopt(handle, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
+	if(curlIPV == 6)
+		curl_easy_setopt(handle, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V6);
 	curl_easy_setopt(handle, CURLOPT_ERRORBUFFER, cerror);
 	curl_easy_setopt(handle, CURLOPT_CONNECTTIMEOUT, mailTimeout);
 	res = setCurlURL(handle, outurl);
@@ -918,7 +930,7 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 	}
 
 	if (debugLevel >= 4)
-		curl_easy_setopt(handle, CURLOPT_VERBOSE, 1);
+		curl_easy_setopt(handle, CURLOPT_VERBOSE, 1l);
 	curl_easy_setopt(handle, CURLOPT_DEBUGFUNCTION, ebcurl_debug_handler);
 	curl_easy_setopt(handle, CURLOPT_DEBUGDATA, &g);
 
@@ -926,17 +938,13 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 		curl_easy_setopt(handle, CURLOPT_USE_SSL, CURLUSESSL_ALL);
 
 	if (account->outssl) {
-		res =
-		    curl_easy_setopt(handle, CURLOPT_USERNAME, account->login);
-		if (res != CURLE_OK) {
-			goto new_handle_cleanup;
+		if(account->login) {
+			    res = curl_easy_setopt(handle, CURLOPT_USERNAME, account->login);
+			if (res != CURLE_OK) goto new_handle_cleanup;
 		}
-
-		res =
-		    curl_easy_setopt(handle, CURLOPT_PASSWORD,
-				     account->password);
-		if (res != CURLE_OK) {
-			goto new_handle_cleanup;
+		if(account->password) {
+			    res = curl_easy_setopt(handle, CURLOPT_PASSWORD, account->password);
+			if (res != CURLE_OK) goto new_handle_cleanup;
 		}
 	}
 
@@ -1009,11 +1017,37 @@ smtp_cleanup:
 	return smtp_success;
 }
 
-/* Send mail to the smtp server. */
+/*********************************************************************
+Send mail to the smtp server.
+recipients is a null terminated array of email addresses.
+attachments is a null terminated array of attachments as filenames
+or as buffer numbers if the attachment is in buffer.
+If nalt is positive it should equal the number of attachments;
+all attachments are alternate presentations.
+If there are automatic attachments in the mail block in .ebrc,
+those might go out as alts as well, this is a bug that no one has run into yet.
+If nfwd is positive it should be 1, forward 1 email,
+which is the first attachment.
+Other attachments are just attachments.
+We should verify the first attachment is a raw email file,
+with headers etc. Use emailTest().
+In this case mime is multipart/related, but if there are then other attachments
+are they related? Should it be mixed, as if they were all just attachments?
+I don't know much about this.
+The whole idea of forwarding the original raw email as a mime component in
+a larger email, they don't seem to do that any more.
+At least my wife's phone doesn't do that.
+It slurps it all up into one html file.
+But then what's the point of forwarding spam to the authorities,
+if they can't see the original headers?
+Well I'm gonna try to do it that way and see what happens.
+This forwarding is rather experimental.
+*********************************************************************/
+
 bool
 sendMail(int account, const char **recipients, const char *body,
 	 int subjat, const char **attachments, const char *refline,
-	 int nalt, bool dosig)
+	 int nalt, int nfwd, bool dosig)
 {
 	char *from, *fromiso, *reply;
 	const struct MACCOUNT *a, *ao, *localMail;
@@ -1039,7 +1073,7 @@ sendMail(int account, const char **recipients, const char *body,
 	ao = a->outssl ? a : localMail;
 	doSignature = dosig;
 
-	nat = 0;		/* number of attachments */
+	nat = 0;		// number of attachments
 	while (attachments[nat])
 		++nat;
 
@@ -1052,11 +1086,6 @@ sendMail(int account, const char **recipients, const char *body,
 	if (nat)
 		mustmime = true;
 
-	if (nalt && nalt < nat) {
-		setError(MSG_AttAlternate);
-		return false;
-	}
-
 	if (!loadAddressBook())
 		return false;
 
@@ -1117,7 +1146,7 @@ sendMail(int account, const char **recipients, const char *body,
 		return false;
 	}
 
-/* verify attachments are readable */
+// verify attachments are readable
 	for (j = 0; (s = attachments[j]); ++j) {
 		if (!ismc && (cx = stringIsNum(s)) >= 0) {
 			if (!cxCompare(cx) || !cxActive(cx, true))
@@ -1202,12 +1231,8 @@ sendMail(int account, const char **recipients, const char *body,
 		stringAndBytes(&out, &j, refline, s - refline);
 		stringAndString(&out, &j, eol);
 	}
-// why should we tell them who we are?
-// I've never received an email that contained User-Agent
-#if 0
 	sprintf(serverLine, "User-Agent: %s%s", currentAgent, eol);
 	stringAndString(&out, &j, serverLine);
-#endif
 	if (subjectLine[0]) {
 		sprintf(serverLine, "Subject: %s%s", subjectLine, eol);
 		stringAndString(&out, &j, serverLine);
@@ -1218,7 +1243,7 @@ sendMail(int account, const char **recipients, const char *body,
 	stringAndString(&out, &j, serverLine);
 
 	if (!mustmime) {
-/* no mime components required, we can just send the mail. */
+// no mime components required, we can just send the mail.
 		sprintf(serverLine,
 			"Content-Type: %s%s%s%sContent-Transfer-Encoding: %s%s%s",
 			ct, charsetString(ct, ce),
@@ -1228,7 +1253,7 @@ sendMail(int account, const char **recipients, const char *body,
 	} else {
 		sprintf(serverLine,
 			"Content-Type: multipart/%s; boundary=%s%sContent-Transfer-Encoding: 7bit%s%s",
-			nalt ? "alternative" : "mixed", boundary, eol, eol,
+			nalt ? "alternative" : nfwd ? "related" : "mixed", boundary, eol, eol,
 			eol);
 		stringAndString(&out, &j, serverLine);
 		stringAndString(&out, &j,
@@ -1251,13 +1276,22 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 	if (mustmime) {
 		for (i = 0; (s = attachments[i]); ++i) {
 			if (!encodeAttachment(s, 0, false, &ct, &ce, &encoded, 0))
-				return false;
+				goto fail;
+			if(nfwd && !i && !stringEqual(ct, "message/rfc822")) {
+				setError(MSG_NotEmail);
+				goto fail;
+			}
 			sprintf(serverLine, "%s--%s%sContent-Type: %s%s", eol,
 				boundary, eol, ct, charsetString(ct, ce));
 			stringAndString(&out, &j, serverLine);
 // If the filename has a quote in it, forget it.
 // Also, suppress filename if this is an alternate presentation.
 			s2 = 0;
+			if (!ismc && (cx = stringIsNum(s)) >= 0) {
+				s = sessionList[cx].lw->f0.fileName;
+// If no file name, call it /, that will be disallowed downstream
+				if(!s) s = "/";
+			}
 			if (!nalt && !strchr(s, '"')
 			    && (ismc || stringIsNum(s) < 0)) {
 // for security reasons, and convenience, don't present the absolute path.
@@ -1289,16 +1323,19 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 			encoded = 0;
 		}		/* loop over attachments */
 
-/* The last boundary */
+// The last boundary
 		sprintf(serverLine, "%s--%s--%s", eol, boundary, eol);
 		stringAndString(&out, &j, serverLine);
 	}
 
-	/* mime format */
-
 	sendmail_success = sendMailSMTP(ao, reply, recipients, out);
 	nzFree(out);
 	return sendmail_success;
+
+fail:
+	nzFree(out);
+	nzFree(encoded);
+	return false;
 }
 
 bool validAccount(int n)
@@ -1324,7 +1361,7 @@ bool sendMailCurrent(int sm_account, bool dosig)
 	char cxbuf[4];
 	int lr, la, ln;
 	char *refline = 0;
-	int nrec, nat, nalt;
+	int nrec, nat, nalt, nfwd;
 	int account = localAccount;
 	int j;
 	bool rc = false;
@@ -1362,7 +1399,7 @@ bool sendMailCurrent(int sm_account, bool dosig)
 	if (!validAccount(account))
 		return false;
 
-	nrec = nat = nalt = 0;
+	nrec = nat = nalt = nfwd = 0;
 
 	recmem = initString(&lr);
 	atmem = initString(&la);
@@ -1410,11 +1447,26 @@ bool sendMailCurrent(int sm_account, bool dosig)
 		}
 
 		if (memEqualCI(line, "attach:", 7)
+		    || memEqualCI(line, "fwd:", 4)
 		    || memEqualCI(line, "alt:", 4)) {
 			if (toupper(line[1]) == 'T')
 				line += 7;
-			else
+			else if (toupper(line[1]) == 'W') {
+				line += 4, ++nfwd;
+				if(nat) {
+						setError(MSG_ForwardFirst);
+					goto done;
+				}
+			} else
 				line += 4, ++nalt;
+			if(nfwd > 1) {
+				setError(MSG_ManyForward);
+				goto done;
+			}
+			if(nfwd && nalt) {
+				setError(MSG_AltForward);
+				goto done;
+			}
 			while (*line == ' ' || *line == '\t')
 				++line;
 			if (*line == '\n') {
@@ -1426,6 +1478,10 @@ bool sendMailCurrent(int sm_account, bool dosig)
 				goto done;
 			}
 			++nat;
+			if (nalt && nalt < nat) {
+				setError(MSG_AttAlternate);
+				goto done;
+			}
 			for (t = line; *t != '\n'; ++t) ;
 			stringAndBytes(&atmem, &la, line, t + 1 - line);
 			continue;
@@ -1487,8 +1543,8 @@ bool sendMailCurrent(int sm_account, bool dosig)
 	atlist[j] = 0;
 
 	sprintf(cxbuf, "%d", context);
-	rc = sendMail(account, reclist, cxbuf, ln, atlist, refline, nalt,
-		      dosig);
+	rc = sendMail(account, reclist, cxbuf, ln, atlist, refline,
+	nalt, nfwd, 		      dosig);
 
 done:
 	nzFree(recmem);
