@@ -182,6 +182,17 @@ char *strmove(char *dest, const char *src)
 	return (char *)memmove(dest, src, strlen(src) + 1);
 }
 
+/* A bit like strncpy but:
+* Don't pad the array after copying the null byte.
+* If we run out of space before the null byte replace the last copied byte with null.
+*/
+char *copyString(char *dest, const char *src, size_t n)
+{
+        if (!memccpy(dest, src, 0, n))
+                dest[n-1] = 0;
+        return dest;
+}
+
 /* OO has a lot of unnecessary overhead, and a few inconveniences,
  * but I really miss it right now.  The following
  * routines make up for the lack of simple string concatenation in C.
@@ -318,19 +329,6 @@ void shiftRight(char *s, char first)
 	s[0] = first;
 }
 
-char *Cify(const char *s, int n)
-{
-	char *u;
-	char *t = allocString(n + 1);
-	if (n)
-		memcpy(t, s, n);
-	for (u = t; u < t + n; ++u)
-		if (*u == 0)
-			*u = ' ';
-	*u = 0;
-	return t;
-}
-
 /* pull a substring out of a larger string,
  * and make it its own allocated string */
 char *pullString(const char *s, int l)
@@ -856,9 +854,9 @@ bool memoryOutToFile(const char *filename, const char *data, int len)
 void truncate0(const char *filename, int fh)
 {
 	if (fh < 0)
-		ignore = truncate(filename, 0l);
+		truncate(filename, 0l);
 	else
-		ignore = ftruncate(fh, 0l);
+		ftruncate(fh, 0l);
 }
 
 long long bufferSizeW(const Window *w, bool browsing)
@@ -993,7 +991,7 @@ static void i_tolower(char *s)
 
 static void i_toupper(char *s)
 {
-	uchar c = *(uchar*)s, d = *(uchar*)(s+1);
+	uchar c = *(uchar*)s;
 	if(!(c&0x80)) {
 		*s = toupper(c);
 		return;
@@ -1342,24 +1340,17 @@ p:
 		case 'p':
 			s = buf + strlen(buf);
 			pwbuf = getpwuid(this_stat.st_uid);
-			if (pwbuf) {
-				l = strlen(pwbuf->pw_name);
-				if (l > 20)
-					l = 20;
-				strncpy(s, pwbuf->pw_name, l);
-				s[l] = 0;
-			} else
+			if (pwbuf)
+// 21 bytes because we want 20 characters plus the null
+				copyString(s, pwbuf->pw_name, 21);
+			else
 				sprintf(s, "%d", this_stat.st_uid);
 			s += strlen(s);
 			*s++ = ' ';
 			grpbuf = getgrgid(this_stat.st_gid);
-			if (grpbuf) {
-				l = strlen(grpbuf->gr_name);
-				if (l > 20)
-					l = 20;
-				strncpy(s, grpbuf->gr_name, l);
-				s[l] = 0;
-			} else
+			if (grpbuf)
+                                copyString(s, grpbuf->gr_name, 21);
+			else
 				sprintf(s, "%d", this_stat.st_gid);
 			s += strlen(s);
 			*s++ = ' ';
@@ -1387,8 +1378,9 @@ p:
 				strcat(buf, "...");
 			else {
 				s = buf + strlen(buf);
-				strncpy(s, newpath, l);
-				s[l] = 0;
+// The value written by readlink is not defined to be null-terminated
+				memcpy(s, newpath, l);
+                                s[l] = 0;
 			}
 			break;
 		}
@@ -1458,7 +1450,7 @@ int getche(void)
 	if(!isInteractive) return getchar();
 	fflush(stdout);
 	ttyRaw(1, 0, true);
-	ignore = read(0, &c, 1);
+	read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
 }
@@ -1469,7 +1461,7 @@ int getch(void)
 	if(!isInteractive) return getchar();
 	fflush(stdout);
 	ttyRaw(1, 0, false);
-	ignore = read(0, &c, 1);
+	read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
 }
@@ -1523,11 +1515,7 @@ char *getFileName(int msg, const char *defname, bool isnew, bool ws)
 			if (!defname)
 				continue;
 // make a copy just to be safe
-			l = strlen(defname);
-			if (l >= ABSPATH)
-				l = ABSPATH - 1;
-			strncpy(buf, defname, l);
-			buf[l] = 0;
+			copyString(buf, defname,  sizeof(buf));
 			p = buf;
 		} else
 			defname = 0;
@@ -1612,17 +1600,22 @@ char *dirSuffix2(int n, const char *path)
 // The return is static, with a limit on path length.
 char *makeAbsPath(const char *f)
 {
-	static char path[ABSPATH + 200];
-	const char *b = cw->baseDirName ? cw->baseDirName : emptyString;
-	if (strlen(b) + strlen(f) > ABSPATH - 2) {
-		setError(MSG_PathNameLong, ABSPATH);
-		return 0;
-	}
-	if(cw->baseDirName)
-		sprintf(path, "%s/%s", b, f);
-	else
-		strcpy(path, f);
-	return path;
+	static char path[ABSPATH];
+        char *ret = NULL;
+
+        if (cw->baseDirName) {
+                int total = snprintf(path, sizeof(path), "%s/%s", cw->baseDirName, f);
+/* I'm not sure if we care about old glibc at this point (probably not) but
+handle both the new-style and old-style truncation checks just in case
+*/
+                if (total < sizeof(path) && total > 0) ret = path;
+        } else if (strlen(f) < sizeof(path)) {
+                strcpy(path, f);
+                ret = path;
+        }
+
+        if (!ret) setError(MSG_PathNameLong, ABSPATH);
+        return ret;
 }
 
 /* loop through the files in a directory */
@@ -1966,6 +1959,10 @@ bool delFiles(int start, int end, bool withtext, char origcmd, char *cmd_p)
 		if (!t)
 			i_printfExit(MSG_NoNlOnDir, file);
 		*t = 0;
+		if(stringEqual(file, "..")) {
+			setError(MSG_DelDotDot);
+			goto abort;
+		}
 		path = makeAbsPath(file);
 		if (!path) {
 abort:
@@ -1996,7 +1993,7 @@ abort:
 				setError(MSG_MetaChar);
 				goto abort;
 			}
-			ignore = asprintf(&a, "rm -rf %c%s%c",
+			asprintf(&a, "rm -rf %c%s%c",
 			qc, path, qc);
 			j = system(a);
 			free(a);
@@ -2029,7 +2026,7 @@ unlink:
 							setError(MSG_MetaChar);
 							goto abort;
 						}
-						ignore = asprintf(&a, "mv -n %c%s%c %c%s%c",
+						asprintf(&a, "mv -n %c%s%c %c%s%c",
 						qc, path, qc, qc, bin, qc);
 						j = system(a);
 						free(a);
@@ -2188,7 +2185,7 @@ bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 						free(path1);
 						return false;
 					}
-					ignore = asprintf(&a, "%s %c%s%c %c%s%c",
+					asprintf(&a, "%s %c%s%c %c%s%c",
 					(origcmd == 'm' ? "mv -n" : "cp -an"),
 					qc, path1, qc, qc, cw2->baseDirName, qc);
 					j = system(a);
@@ -2322,8 +2319,7 @@ static bool envExpand(const char *line, const char **expanded)
 	}
 
 	udir = 0;
-	strncpy(var1, line + 1, l);
-	var1[l] = 0;
+	copyString(var1, line + 1, l + 1);
 	if (l) {
 		pw = getpwnam(var1);
 		if (!pw) {
@@ -2482,17 +2478,18 @@ char *envFileAlloc(const char *line)
 bool envFileDown(const char *line, const char **expanded)
 {
 	static char line2[MAXTTYLINE];
+        int total;
 
 	if (!downDir || strchr(line, '/'))
 /* we don't necessarily expect there to be a file here */
 		return envFile(line, expanded);
 
-	if (strlen(downDir) + strlen(line) >= sizeof(line2) - 1) {
+        total = snprintf(line2, sizeof(line2), "%s/%s", downDir, line);
+	if (total == sizeof(line2) || total < 0) {
 		setError(MSG_ShellLineLong);
 		return false;
-	}
+        }
 
-	sprintf(line2, "%s/%s", downDir, line);
 	*expanded = line2;
 	return true;
 }
